require 'spec_helper'
require 'interfaces/sqs'

describe 'SQS', :sqs do

  SQS_CONFIG = {
    access_key_id:      'fake access key',
    secret_access_key:  'fake secret key',
    sqs_queues: %w(q1 q2),
    batch_size: 3
  }

  before :all do
    @sqs = AWS::SQS.new(SQS_CONFIG)

    @q1 = @sqs.queues.create('q1')
    @q2 = @sqs.queues.create('q2')

    MailHandler::SQS.setup(SQS_CONFIG)
  end

  let(:email) do
    {
      from: 'email@email.com',
      to: 'email2@email.com',
      subject: 'testing',
      text: 'testing text',
      reply_to: 'noreply@email.com'
    }
  end

  describe '#fetch_messages' do
    subject do
      MailHandler::SQS.client.fetch_messages.map do |msg|
        JSON.parse(msg.body, symbolize_names: true)
      end
    end

    context 'when no messages' do
      it { is_expected.to be_empty }
    end

    context 'when one message' do
      describe 'from one queue' do
        before { @q1.send_message(JSON.dump(email)) }

        it { is_expected.to eq [email] }
      end

      describe 'from multiple queues' do
        before { [@q1, @q2].each { |q| q.send_message(JSON.dump email) } }

        it { is_expected.to eq [email, email] }
      end
    end

    context 'when multiple messages' do
      describe 'from one queue' do
        before { 4.times { @q1.send_message(JSON.dump email) } }

        it { is_expected.to eq 3.times.map { email } }
      end

      describe 'from multiple queues' do
        before do
          [@q1, @q2].each { |q| 4.times { q.send_message(JSON.dump email) } }
        end

        it { is_expected.to eq 6.times.map { email } }
      end
    end
  end

  describe 'deleting messages' do
    subject { MailHandler::SQS.client.fetch_messages }

    before { @q1.send_message(JSON.dump email) }

    context 'when not yet deleted' do
      it { is_expected.not_to be_empty }
    end

    context 'when deleted' do
      before { MailHandler::SQS.client.fetch_messages.first.delete }

      it { is_expected.to be_empty }
    end

  end
end
